require 'time'
require 'rubygems'
require 'nokogiri'
require 'open-uri'
require 'yaml'
require 'zlib'
require 'csv'

CACHE_PREFIX = 'cache/'
OUT_PREFIX   = 'out/'
MIN_TURNS    = 6

# ------------
# LADDER GAMES
# ------------
puts "\033[33m>>>\033[0m parsing \033[33mladders\033[0m"
ladder_cache = CACHE_PREFIX + 'ladders.yml'

# if cache file doesnt exitst
unless File.exists?(ladder_cache)
  ladder_list = []
  ladder_now  = Time.now + 30 * 3600
  ladder_min  = Time.parse('2009-04-02')
  
  while ladder_now > ladder_min do
    results_link = "http://ladder.subversiva.org/gamehistory.php?reporteddirection=%3C%3D&reportdate=#{ladder_now.strftime('%Y-%m-%d')}"
    puts " ** #{results_link}"
    
    Nokogiri::HTML(open(results_link)).css('#games tbody tr').each do |game|
      begin
        reported   = Time.parse(game.at_xpath('td[1]').content)
        winner     = game.at_xpath('td[2]/a').content
        winner_elo = game.at_xpath('td[4]').content.to_i
        loser      = game.at_xpath('td[3]/a').content
        loser_elo  = game.at_xpath('td[5]').content.to_i
        replay     = game.at_xpath('td[11]/a') ? true : false
      rescue
        nil
      else
        ladder_list << [reported, winner, winner_elo, loser, loser_elo, replay]
      end
    end
    
    ladder_now -= 5 * 24 * 3600
    ladder_list.uniq!
    sleep(1)
  end
  
  ladder_list.collect! {|game| {:reported => game[0], :winner => game[1], :winner_elo => game[2], :loser => game[3], :loser_elo => game[4], :replay => game[5]}}
  File.open(ladder_cache, 'w') {|f| f.write(YAML::dump(ladder_list))}
# if cache was found
else
  puts ' ** ladders cache found'
  ladder_list = YAML::load(open(ladder_cache))
end

# -------------
# ARCHIVE GAMES
# -------------
puts "\033[33m>>>\033[0m parsing \033[33marchives\033[0m"
archive_cache = CACHE_PREFIX + 'archives.yml'

# if cache file doesnt exitst
unless File.exists?(archive_cache)
  archive_list = []
  archive_link = 'http://replays.wesnoth.org/1.6/'
  
  dirs_exp = '//tr/td[2]/a'
  game_exp = '//tr'
  
  Nokogiri::HTML(open(archive_link)).xpath(dirs_exp).each do |directory|
    next unless /\d+\// =~ directory.content
    puts " ** #{archive_link + directory.content}"
    
    Nokogiri::HTML(open(archive_link + directory.content)).xpath(game_exp).each do |game|
      if /(2p_-_(.+?)_Turn_(\d+)_\(\d+\)\.gz)/m =~ game
        filename, map, turn = $1, $2, $3
        
        filename = archive_link + directory + filename
        map      = map.gsub('_', ' ')
        turn     = turn.to_i
        
        saved    = Time.parse(game.at_xpath('td[3]').content)
        players  = game.at_xpath('td[5]').content.gsub(/.*players:/m, '').strip.split(', ').uniq
        
        players.delete('ai')
        next unless players.size == 2 and turn > MIN_TURNS
        
        archive_list << {:saved => saved, :map => map, :turn => turn, :link => filename, :player_1 => players[0], :player_2 => players[1]}
      end
    end
    
    sleep(1)
  end
  File.open(archive_cache, 'w') {|f| f.write(YAML::dump(archive_list))}
# if cache was found
else
  puts ' ** archives cache found'
  archive_list = YAML::load(open(archive_cache))
end

# -------------
# COMPARE GAMES
# -------------
puts "\033[33m>>>\033[0m compare \033[33mladders\033[0m with \033[33marchives\033[0m"
results = []

@parsed = 0
@errors = 0

# this one do the replay riping
def get_all_game_data(l_game, a_game)
  begin
    sides = []
    
    regex_cache = /.*((\d{8,})\/.*)/
    file_cache  = CACHE_PREFIX + 'replay_archive/' + regex_cache.match(a_game[:link])[1]
    dir_cache   = CACHE_PREFIX + 'replay_archive/' + regex_cache.match(a_game[:link])[2]
    
    Dir.mkdir(dir_cache) unless File.directory?(dir_cache)
    unless File.exist?(file_cache)
      puts " -> replay from \033[34marchive\033[0m, link: #{a_game[:link]}"
      File.open(file_cache, 'w') {|f| f.write(open(a_game[:link]).read)}
    else
      puts " -> replay from \033[34mcache\033[0m, link: #{a_game[:link]}"
    end
    
    replay    = Zlib::GzipReader.new(open(file_cache)).read
    regex_era = /\[era\].*?^\s*name=\s*_\s*"Default".*?\[multiplayer_side\]/m
    
    raise 'Invalid era' unless regex_era.match(replay)
    
    # can be fixed to parse corrupted replays
    regex_sides = /^\s*\[side\](.+)\[\/side\]/m
    regex_clean = /\[(ai|unit|village)\].+?\[\/(ai|unit|village)\]/m
    regex_split = /\[\/side\].*?\[side\]/m
    
    regex_sides.match(replay)[1].gsub(regex_clean, '').split(regex_split).each do |side_data|
      regex_side    = /^\s*side="(1|2)"/m
      regex_player  = /^\s*name="(.+?)"/m
      regex_leader  = /^\s*type="(.+?)"/m
      regex_faction = /^\s*recruit="(.+?)"/m
      
      next unless regex_side.match(side_data)
      
      side    = regex_side.match(side_data)[1].to_i
      player  = regex_player.match(side_data)[1]
      leader  = regex_leader.match(side_data)[1]
      faction = case regex_faction.match(side_data)[1]
        when 'Drake Burner,Drake Clasher,Drake Glider,Drake Fighter,Saurian Skirmisher,Saurian Augur'                       : 'Drakes'
        when 'Cavalryman,Horseman,Spearman,Fencer,Heavy Infantryman,Bowman,Mage,Merman Fighter'                             : 'Loyalists'
        when 'Elvish Fighter,Elvish Archer,Mage,Elvish Shaman,Elvish Scout,Wose,Merman Hunter'                              : 'Rebels'
        when 'Dwarvish Guardsman,Dwarvish Fighter,Dwarvish Ulfserker,Dwarvish Thunderer,Thief,Poacher,Footpad,Gryphon Rider': 'Knalgan Alliance'
        when 'Orcish Grunt,Troll Whelp,Wolf Rider,Orcish Archer,Orcish Assassin,Naga Fighter,Goblin Spearman'               : 'Northerners'
        when 'Skeleton,Skeleton Archer,Walking Corpse,Ghost,Vampire Bat,Dark Adept,Ghoul'                                   : 'Undead'
      end
      
      sides[side - 1] = {:side => side, :player => player, :faction => faction, :leader => leader}
    end

    raise 'Invalid sides' unless sides.size == 2
  rescue
    puts " -- \033[31mfail\033[0m"
    @errors += 1
    
    sides =  []
  else
    puts " ++ \033[32mdone\033[0m"
    @parsed += 1
    
    first_player_won = /#{l_game[:winner]}/i =~ sides[0][:player] ? true : false
    return {
      :reported   => Time.parse(l_game[:reported].strftime('%Y-%m-%d %H:%M:%S UTC')),
      :replay     => l_game[:replay],
      :winner     => l_game[:winner],
      :winner_elo => l_game[:winner_elo],
      :loser      => l_game[:loser],
      :loser_elo  => l_game[:loser_elo],
      :saved      => Time.parse(a_game[:saved].strftime('%Y-%m-%d %H:%M:%S UTC')),
      :link       => a_game[:link],
      :map        => a_game[:map],
      :turn       => a_game[:turn],
      :winner_an      => first_player_won ? a_game[:player_1] : a_game[:player_2],
      :loser_an       => first_player_won ? a_game[:player_2] : a_game[:player_1],
      :winner_side    => first_player_won ? sides[0][:side] : sides[1][:side],
      :loser_side     => first_player_won ? sides[1][:side] : sides[0][:side],
      :winner_faction => first_player_won ? sides[0][:faction] : sides[1][:faction],
      :loser_faction  => first_player_won ? sides[1][:faction] : sides[0][:faction],
      :winner_leader  => first_player_won ? sides[0][:leader] : sides[1][:leader],
      :loser_leader   => first_player_won ? sides[1][:leader] : sides[0][:leader],
    }
  end
  {}
end

# finally data
before_limit = 45 * 60
after_limit  = 105 * 60

ladder_list.each do |ladder_game|
  if ladder_game[:reported] >= Time.parse('2009-04-03 03:00:00')
    
    puts "\n ** \033[34mREPORTED:\033[0m #{ladder_game[:reported].strftime('%Y-%m-%d %H:%M')} (LADDER TIME) \033[34mBY:\033[0m #{ladder_game[:winner]} \033[34mVS:\033[0m #{ladder_game[:loser]}"
    errors_before = @errors
    result       = {}
    
    archive_list.reverse_each do |archive_game|
      if /#{ladder_game[:winner]}|#{ladder_game[:loser]}/i =~ archive_game[:player_1] and /#{ladder_game[:winner]}|#{ladder_game[:loser]}/i =~ archive_game[:player_2]
        
        # from 2009-10-25 01:20:00 to present
        if (ladder_game[:reported] >= Time.parse('2009-10-25 01:20:00')) and (archive_game[:saved] >= ladder_game[:reported] + before_limit) and (archive_game[:saved] <= ladder_game[:reported] + after_limit)
          result = get_all_game_data(ladder_game, archive_game)
          break
        # from 2009-09-02 06:40:00 to 2009-10-25 01:20:00
        elsif (ladder_game[:reported] >= Time.parse('2009-09-02 06:40:00')) and (ladder_game[:reported] < Time.parse('2009-10-25 01:20:00')) and (archive_game[:saved] >= ladder_game[:reported] + before_limit + 3600) and (archive_game[:saved] <= ladder_game[:reported] + after_limit + 3600)
          result = get_all_game_data(ladder_game, archive_game)
          break
        # from 2009-05-30 07:50:00 to 2009-09-02 06:40:00
        elsif (ladder_game[:reported] >= Time.parse('2009-05-30 07:50:00')) and (ladder_game[:reported] < Time.parse('2009-09-02 06:40:00')) and (archive_game[:saved] >= ladder_game[:reported] + before_limit + 8 * 3600) and (archive_game[:saved] <= ladder_game[:reported] + after_limit + 8 * 3600)
          result = get_all_game_data(ladder_game, archive_game)
          break
        # from 2009-04-12 22:00:00 to 2009-05-30 07:50:00
        elsif (ladder_game[:reported] >= Time.parse('2009-04-12 22:00:00')) and (ladder_game[:reported] < Time.parse('2009-05-30 07:50:00')) and (archive_game[:saved] >= ladder_game[:reported] + before_limit + 7 * 3600 + 1800) and (archive_game[:saved] <= ladder_game[:reported] + after_limit + 7 * 3600 + 1800)
          result = get_all_game_data(ladder_game, archive_game)
          break
        # from 2009-04-07 11:45:00 to 2009-04-12 22:00:00
        elsif (ladder_game[:reported] >= Time.parse('2009-04-07 11:45:00')) and (ladder_game[:reported] < Time.parse('2009-04-12 22:00:00')) and (archive_game[:saved] >= ladder_game[:reported] + before_limit + 6 * 3600) and (archive_game[:saved] <= ladder_game[:reported] + after_limit + 6 * 3600)
          result = get_all_game_data(ladder_game, archive_game)
          break
        # before 2009-04-07 11:45:00
        elsif (ladder_game[:reported] < Time.parse('2009-04-07 11:45:00')) and (archive_game[:saved] >= ladder_game[:reported] + before_limit + 6 * 3600 + 1800) and (archive_game[:saved] <= ladder_game[:reported] + after_limit + 6 * 3600 + 1800)
          result = get_all_game_data(ladder_game, archive_game)
          break
        end
        
      end
    end
    
    unless result.empty?
      results << result
      
      puts " -> LADDER URL: http://ladder.subversiva.org/gamedetails.php?reported_on=#{result[:reported].strftime('%Y-%m-%d+%H:%M:%S')}"
      puts " -> DETAILS:    #{result[:saved].strftime('%Y-%m-%d %H:%M')} (GMT+1), #{result[:map]}, #{result[:turn]}"
      puts " -> WINNER:     #{result[:winner_an]}, #{result[:winner_elo]}, #{result[:winner_side]}, #{result[:winner_faction]}, #{result[:winner_leader]}"
      puts " -> LOSER:      #{result[:loser_an]}, #{result[:loser_elo]}, #{result[:loser_side]}, #{result[:loser_faction]}, #{result[:loser_leader]}"
    else
      puts " ?? \033[31mno matches\033[0m" if errors_before == @errors
    end
  
  # sleep(1)  
  end
end

# -------
# SUMMARY
# -------
puts "\n\033[33m>>>\033[0m results \033[33moverview\033[0m"

puts " ** ladder games: #{ladder_list.size}"
puts " ** inspected: #{@parsed}"
puts " ** corrupted: #{@errors}"
puts " ** missing: #{ladder_list.size - (@parsed + @errors)}"

# ----
# DUMP
# ----
puts "\033[33m>>>\033[0m dumping \033[33mdata\033[0m"

puts ' ** to yaml'
File.open(OUT_PREFIX + 'results.yml', 'w') {|f| f.write(YAML::dump(results))}

puts ' ** to csv'
out = File.open(OUT_PREFIX + 'results.csv', 'w')

CSV::Writer.generate(out) {|csv|
  results.each {|result|
    csv << [
      result[:reported].strftime('%Y-%m-%d %H:%M:%S'), # reported on ladder (time)
      result[:replay],                                 # if it has replay on ladder website
      result[:saved].strftime('%Y-%m-%d %H:%M:%S'),    # time when replay was saved in replay archive
      result[:link],                                   # link to replay in archive
      result[:map],                                    # map on what game was played 
      result[:turn],                                   # how many turns game had
      result[:winner],                                 # winner ladder nickname
      result[:winner_an],                              # winner replay archive nickname
      result[:winner_elo],                             # winner elo
      result[:winner_side],                            # winner side
      result[:winner_faction],                         # winner faction
      result[:winner_leader],                          # winner leader
      result[:loser],                                  # loser ladder nickname
      result[:loser_an],                               # loser replay archive nickname
      result[:loser_elo],                              # loser elo
      result[:loser_side],                             # loser side
      result[:loser_faction],                          # loser faction
      result[:loser_leader]                            # loser leader
    ]
  }
}

out.close
