require 'rubygems'
require 'prawn'
require 'prawn/layout'
require 'yaml'

puts "\033[33m>>>\033[0m generating \033[33mpdfs\033[0m"

# generate pdf for each array element
# [min_elo, min_turns, max_turns]
[
  [1000, 7, 70],
  [1000, 7, 14],
  [1000, 15, 21],
  [1000, 22, 70],
  [1800, 7, 70],
  [1800, 7, 14],
  [1800, 15, 21],
  [1800, 22, 70]
].each do |c|
  puts " ** min elo, min turns, max turns: #{c[0]}, #{c[1]}, #{c[2]}"
  regex_map = /Caves of the Basilisk|Cynsaun Battlefield|Den of Onis|Fallenstar Lake|Hamlets|Hornshark Island|Howling Ghost Badlands|Sablestone Delta|Silverhead Crossing|Sulla's Ruins|The Freelands|Weldyn Channel/
  
  # filter results array
  results   = []
  YAML::load(File.open('out/results.yml', 'r')).each do |result|
    results << result if (result[:winner_elo] >= c[0]) and (result[:loser_elo] >= c[0]) and (result[:turn] >= c[1]) and (result[:turn] <= c[2]) and (regex_map =~ result[:map])
  end
  
  # set variables
  all_games  = results.size
  row_colors = ['ffffff', 'dddddd']
  
  # new pdf
  pdf = Prawn::Document.new
  
  # document info
  pdf.font_size 14
  pdf.text "
   #{regex_map.to_s.gsub(/\(\?-mix:|\)/, '').gsub('|', ' / ')}
   
   - generated on #{Time.now.utc}
   - both players have at least #{c[0]} ELO
   - game length from #{c[1]} to #{c[2]} turns
   
   + #{all_games} games found
  "
  pdf.font_size 10
  
  # -------------
  # side overview
  # -------------
  sides = [
    ['Player 1', 0, nil],
    ['Player 2', 0, nil]
  ]
  
  results.each do |result|
    if result[:winner_side] == 1
      sides[0][1] += 1
    else
      sides[1][1] += 1
    end
  end
  
  sides.each do |side|
    side[2] = "%.2f" % (side[1] / all_games.to_f * 100)
  end
  
  pdf.text "\nSide overview:"
  pdf.table sides, :headers => ['Side', 'Games won', 'Percent won'], :row_colors => row_colors, :font_size => 8
  
  # ------------
  # map overview
  # ------------
  maps  = [
    ["Caves of the Basilisk",  0, nil, 0],
    ["Cynsaun Battlefield",    0, nil, 0],
    ["Den of Onis",            0, nil, 0],
    ["Fallenstar Lake",        0, nil, 0],
    ["Hamlets",                0, nil, 0],
    ["Hornshark Island",       0, nil, 0],
    ["Howling Ghost Badlands", 0, nil, 0],
    ["Sablestone Delta",       0, nil, 0],
    ["Silverhead Crossing",    0, nil, 0],
    ["Sulla's Ruins",          0, nil, 0],
    ["The Freelands",          0, nil, 0],
    ["Weldyn Channel",         0, nil, 0],
    ["Other",                  0, nil, 0]
  ]
  
  results.each do |result|
    if result[:map] =~ /Caves of the Basilisk/
      maps[0][1] += 1
      maps[0][3] += result[:turn]
    elsif result[:map] =~ /Cynsaun Battlefield/
      maps[1][1] += 1
      maps[1][3] += result[:turn]
    elsif result[:map] =~ /Den of Onis/
      maps[2][1] += 1
      maps[2][3] += result[:turn]
    elsif result[:map] =~ /Fallenstar Lake/
      maps[3][1] += 1
      maps[3][3] += result[:turn]
    elsif result[:map] =~ /Hamlets/
      maps[4][1] += 1
      maps[4][3] += result[:turn]
    elsif result[:map] =~ /Hornshark Island/
      maps[5][1] += 1
      maps[5][3] += result[:turn]
    elsif result[:map] =~ /Howling Ghost Badlands/
      maps[6][1] += 1
      maps[6][3] += result[:turn]
    elsif result[:map] =~ /Sablestone Delta/
      maps[7][1] += 1
      maps[7][3] += result[:turn]
    elsif result[:map] =~ /Silverhead Crossing/
      maps[8][1] += 1
      maps[8][3] += result[:turn]
    elsif result[:map] =~ /Sulla's Ruins/
      maps[9][1] += 1
      maps[9][3] += result[:turn]
    elsif result[:map] =~ /The Freelands/
      maps[10][1] += 1
      maps[10][3] += result[:turn]
    elsif result[:map] =~ /Weldyn Channel/
      maps[11][1] += 1
      maps[11][3] += result[:turn]
    else
      maps[12][1] += 1
      maps[12][3] += result[:turn]
    end
  end
  
  maps.each do |map|
    map[2] = "%.2f" % (map[1] / all_games.to_f * 100)
    map[3] = map[3] / map[1] unless map[1] == 0
  end
  
  pdf.text "\nMap overview:"
  pdf.table maps, :headers => ['Map', 'Played games', 'Played percent', 'Game length (avg)'], :row_colors => row_colors, :font_size => 8
  
  # ----------------
  # faction overview
  # ----------------
  factions = [
    ['Drakes',           0, nil, nil, 0, nil, nil, 0],
    ['Loyalists',        0, nil, nil, 0, nil, nil, 0],
    ['Rebels',           0, nil, nil, 0, nil, nil, 0],
    ['Knalgan Alliance', 0, nil, nil, 0, nil, nil, 0],
    ['Northerners',      0, nil, nil, 0, nil, nil, 0],
    ['Undead',           0, nil, nil, 0, nil, nil, 0]
  ]
  
  results.each do |result|
    if    result[:winner_faction] == 'Drakes'           then factions[0][1] += 1
    elsif result[:winner_faction] == 'Loyalists'        then factions[1][1] += 1
    elsif result[:winner_faction] == 'Rebels'           then factions[2][1] += 1
    elsif result[:winner_faction] == 'Knalgan Alliance' then factions[3][1] += 1
    elsif result[:winner_faction] == 'Northerners'      then factions[4][1] += 1
    elsif result[:winner_faction] == 'Undead'           then factions[5][1] += 1
    end
    
    if    result[:loser_faction] == 'Drakes'           then factions[0][4] += 1
    elsif result[:loser_faction] == 'Loyalists'        then factions[1][4] += 1
    elsif result[:loser_faction] == 'Rebels'           then factions[2][4] += 1
    elsif result[:loser_faction] == 'Knalgan Alliance' then factions[3][4] += 1
    elsif result[:loser_faction] == 'Northerners'      then factions[4][4] += 1
    elsif result[:loser_faction] == 'Undead'           then factions[5][4] += 1
    end
  end
  
  factions.each do |faction|
    faction[7] = faction[1] + faction[4]
    
    faction[2] = "%.2f" % (faction[1] / all_games.to_f * 100)
    faction[3] = "%.2f" % (faction[1] / faction[7].to_f * 100)
    faction[5] = "%.2f" % (faction[4] / all_games.to_f * 100)
    faction[6] = "%.2f" % (faction[4] / faction[7].to_f * 100)
  end
  
  pdf.text "\nFaction overview:"
  pdf.table factions, :headers => ['Faction', 'Games won', 'Percent won', 'Success rate', 'Games lost', 'Percent lost', 'Failure rate', 'Times played'], :row_colors => row_colors, :font_size => 8
  
  # ----------------
  # matchup overview
  # ----------------
  matchups = []
  
  results.each do |result|
    winner = "#{result[:winner_faction]} (P#{result[:winner_side]})"
    loser  = "#{result[:loser_faction]} (P#{result[:loser_side]})"
    
    matchup       = [winner, loser].sort
    from_matchups = matchups.select{|m| (matchup[0] == m[0]) and (matchup[1] == m[2])}
    matchup       = from_matchups.empty? ? matchup.insert(1, 0).insert(3, 0) : from_matchups.first ; matchups.delete(from_matchups.first)
    
    matchup[0] == winner ? matchup[1] += 1 : matchup[3] += 1
    matchups << matchup
  end
  
  matchups.sort! {|m1, m2| m1[0] + m1[2] <=> m2[0] + m2[2]}
  
  matchups.each do |matchup|
    all_games = matchup[1] + matchup[3]
    
    matchup.insert(2, "%.2f" % (matchup[1] / all_games.to_f * 100))
    matchup.insert(5, "%.2f" % (matchup[4] / all_games.to_f * 100))
  end
  
  pdf.start_new_page
  pdf.text "\nMatchup overview:"
  pdf.table matchups, :headers => ['Faction', 'Wins', 'Percent', 'Faction', 'Wins', 'Percent'], :row_colors => row_colors, :font_size => 6
  
  # ----------------------
  # render file in pdf dir
  # ----------------------
  pdf.render_file("pdf/min_elo_#{c[0]}_turns_#{c[1]}-#{c[2]}.pdf")

# next criteria or finish
end
